package friend_circle

import (
	"applet/app/db"
	"applet/app/e"
	md "applet/app/md/friend_circle"
	"applet/app/svc"
	svc2 "applet/app/svc/friend_circle"
	"applet/app/utils"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
	"github.com/gin-gonic/gin"
	"time"
)

// GetBlackList
// @Summary      社交管理-动态设置-黑名单(获取)
// @Tags         动态设置
// @Description  黑名单(获取)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param    limit    query	 int 	true  "每页大小"
// @Param    page    query	 int 	true  "页数"
// @Param    phone    query	 int 	true  "手机号"
// @Param    nickname    query	 int 	true  "用户名"
// @Param    uid    query	 int 	true  "用户id"
// @Success      200   {object}   	md.GetBlackListResp "具体数据"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router        /api/friendCircleSettings/blackList/index [GET]
func GetBlackList(c *gin.Context) {
	pageStr := c.DefaultQuery("page", "1")
	limitStr := c.DefaultQuery("limit", "10")
	phone := c.Query("phone")
	nickname := c.Query("nickname")
	uid := c.Query("uid")

	page := utils.StrToInt(pageStr)
	limit := utils.StrToInt(limitStr)

	blackList, total, err2 := svc2.GetBlackList(page, limit, phone, nickname, uid)
	if err2 != nil {
		e.OutErr(c, e.ERR_DB_ORM, err2.Error())
		return
	}

	adminIds := make([]int, 0, len(blackList))
	for _, list := range blackList {
		adminIds = append(adminIds, list.AdmId)
	}
	adminDb := implement.NewAdminDb(db.Db)
	admins, err := adminDb.AdminFindByParams(map[string]interface{}{
		"key":   "adm_id",
		"value": adminIds,
	})
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, nil)
		return
	}
	adminMap := make(map[int]model.Admin, len(admins))
	for _, admin := range admins {
		adminMap[admin.AdmId] = admin
	}
	list := make([]md.GetBlackListNode, 0, len(blackList))
	for _, black := range blackList {
		node := md.GetBlackListNode{
			BlackListId:   black.Id,
			Memo:          black.Memo,
			Uid:           black.Uid,
			UserNickname:  black.UserNickname,
			AdminId:       black.AdmId,
			AdminNickname: adminMap[black.AdmId].Username,
			CreatedAt:     black.CreateAt,
		}
		list = append(list, node)
	}
	resp := md.GetBlackListResp{
		List: list,
		Paginate: md.Paginate{
			Limit: limit,
			Page:  page,
			Total: total,
		},
	}
	e.OutSuc(c, resp, nil)
}

// AddBlackList
// @Summary      社交管理-动态设置-黑名单(新增)
// @Tags         动态设置
// @Description  黑名单(新增)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param    req    body	md.AddBlackListReq 	true  "用户id 必填"
// @Success      200   {string}   	"生成的黑名单id"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router        /api/friendCircleSettings/blackList/add [POST]
func AddBlackList(c *gin.Context) {
	var req md.AddBlackListReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = svc.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}

	admin := svc.GetUser(c)
	now := time.Now()
	blackList := model.EggFriendCircleUserBlackList{
		Uid:      req.Uid,
		AdmId:    admin.AdmId,
		Memo:     req.Memo,
		CreateAt: now.Format("2006-01-02 15:04:05"),
		UpdateAt: now.Format("2006-01-02 15:04:05"),
	}
	blackListDb := implement.NewEggFriendCircleUserBlackListDb(db.Db)
	blackListId, err := blackListDb.EggFriendCircleUserBlackListInsert(&blackList)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, nil)
		return
	}
	e.OutSuc(c, blackListId, nil)
}

// DeleteBlackList
// @Summary      社交管理-动态设置-黑名单(删除)
// @Tags         动态设置
// @Description  黑名单(删除)
// @Accept       json
// @Produce      json
// @param Authorization header string true "验证参数Bearer和token空格拼接"
// @Param    req    body	md.DeleteBlackListReq 	true  "黑名单id 必填"
// @Success      200   {string}   	"删除数据数量"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router        /api/friendCircleSettings/blackList/del [DELETE]
func DeleteBlackList(c *gin.Context) {
	var req md.DeleteBlackListReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = svc.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}
	blackListDb := implement.NewEggFriendCircleUserBlackListDb(db.Db)
	affected, err := blackListDb.EggFriendCircleUserBlackListDeleteById(req.BlackListId)
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, nil)
		return
	}
	e.OutSuc(c, affected, nil)
}
